import re

from cy_ioc_extract.constants import ALL_EXTRACT_FIELDS, ERRORS, ALL_REGEX
from cy_ioc_extract.exception import UnsupportedExtractFieldError


class IOCEXtract:
    def __init__(self, raw_text, extract_fields=None, validate_ioc=True):
        self.validate = validate_ioc
        self.raw_text = raw_text
        self.extract_fields = extract_fields or ALL_EXTRACT_FIELDS
        self.validate_extract_fields(self.extract_fields)

    def extract_ioc(self):
        """
        Extract IOC from raw text.
        Also validate(currently validation is only applied on domains) if required.
        """
        extracted_data = {ioc_type: list(set(re.findall(ALL_REGEX[ioc_type]["regex"], self.raw_text, flags=re.IGNORECASE))) for ioc_type in self.extract_fields}
        for ioc_type in self.extract_fields:
            custom_extract_function = ALL_REGEX[ioc_type].get("custom_extract_function")
            if custom_extract_function:
                # Some IOC types are extracted with custom function
                # Example, URLs - Regex for this is combination of two regex
                # And hence findall will give group of matches
                extracted = custom_extract_function(self.raw_text)
            else:
                regex = ALL_REGEX[ioc_type]["regex"]
                extracted = list(set(re.findall(regex, self.raw_text, flags=re.IGNORECASE)))
            # Check if IOCs has to be validated or not
            # domains are usually validated against ICANN and OpenNIC TDL standard
            validator = ALL_REGEX[ioc_type].get("validator")
            if self.validate and validator and extracted:
                extracted = validator(extracted)
            extracted_data[ioc_type] = extracted
        return extracted_data

    @staticmethod
    def validate_extract_fields(extract_fields):
        invalid_items = set(extract_fields) - set(ALL_EXTRACT_FIELDS)
        if invalid_items:
            raise UnsupportedExtractFieldError(
                ERRORS.get("invalid_field_to_extract") % (str(invalid_items), str(set(ALL_EXTRACT_FIELDS))))
